import json
from datetime import datetime

from portal.database.models import User, Submission, Role, Course, Group, Project, ProjectConfig


def assert_user(expected: User, actual: dict):
    assert expected.id == actual['id']
    assert expected.uco == actual['uco']
    assert expected.email == actual['email']
    assert expected.username == actual['username']
    assert expected.name == actual['name']
    assert expected.is_admin == actual['is_admin']


def compare_user(expected: User, actual: dict):
    return expected.id == actual['id'] and expected.uco == actual['uco'] and expected.email == actual['email']


def assert_user_in(user_list: list, user: dict):
    for u in user_list:
        if compare_user(u, user):
            return True
    return False


def assert_submission(expected: Submission, actual: dict):
    assert expected.id == actual['id']
    assert expected.state.name == actual['state']
    assert expected.project.course_id == actual['project']["course"]["id"]
    assert expected.scheduled_for == actual['scheduled_for']
    assert expected.project_id == actual['project']['id']


def compare_submission(expected: Submission, actual: dict):
    return expected.id == actual['id'] \
           and expected.state == actual['state'] \
           and expected.project.id == actual['project']['id'] \
           and expected.scheduled_for == actual['scheduled_for']


def assert_submission_in(submission_list: list, submission: dict):
    for s in submission_list:
        if compare_submission(s, submission):
            return True
    return False


def assert_role(expected: Role, actual: dict):
    assert expected.id == actual['id']
    assert expected.name == actual['name']
    assert expected.description == actual['description']
    assert expected.course_id == actual['course']['id']


def assert_course(expected: Course, actual: dict):
    assert expected.id == actual['id']
    assert expected.name == actual['name']
    assert expected.codename == actual['codename']


def compare_course(expected: Course, actual: dict):
    return expected.id == actual['id'] and expected.codename == actual['codename']


def assert_course_in(course_list: list, course: dict):
    for c in course_list:
        if compare_course(c, course):
            return True
    return False


def assert_group(expected: Group, actual: dict):
    assert expected.id == actual['id']
    assert expected.name == actual['name']
    assert expected.course_id == actual['course']['id']


def compare_group(expected: Group, actual: dict):
    return expected.id == actual['id'] and expected.name == actual['name'] \
           and expected.course_id == actual['course']['id']


def assert_group_in(group_list: list, group: dict):
    for g in group_list:
        if compare_group(g, group):
            return True
    return False


def compare_project(expected: Project, actual: dict):
    return expected.id == actual['id'] and expected.name == actual['name'] \
           and expected.course_id == actual['course']['id']


def assert_project(expected: Project, actual: dict):
    assert expected.id == actual['id']
    assert expected.name == actual['name']
    assert expected.course_id == actual['course']['id']


def assert_project_in(project_list: list, project: dict):
    for p in project_list:
        if compare_project(p, project):
            return True
    return True


def assert_project_config(expected: ProjectConfig, actual: dict):
    assert expected.id == actual['id']
    assert expected.project.id == actual['project']['id']
    assert expected.test_files_source == actual['test_files_source']
    assert expected.file_whitelist == actual['file_whitelist']
    assert expected.pre_submit_script == actual['pre_submit_script']
    assert expected.post_submit_script == actual['post_submit_script']
    assert expected.submission_parameters == actual['submission_parameters']
    assert expected.submission_scheduler_config == actual['submission_scheduler_config']
    assert expected.submissions_allowed_from == actual['submissions_allowed_from']
    assert expected.submissions_allowed_to == actual['submissions_allowed_to']
    assert expected.archive_from == actual['archive_from']


# source: https://stackoverflow.com/questions/11875770/how-to-overcome-datetime-datetime-not-json-serializable
class DateTimeEncoder(json.JSONEncoder):
    def default(self, o):
        if isinstance(o, datetime):
            return o.isoformat()

        return json.JSONEncoder.default(self, o)
